/*
 * INNOV-2: Build a Better Web Interface Using AJAX
 *
 * ajaxorders.p
 *
 * This program generates an xml response to an AJAX request for a list of
 * orders for a specific customer.
 *
 * Note that the xml responses must be sent with a text/xml content type.
 * If you are using WebSpeed, then you should set the content type with the
 * OutputContentType procedure provided by Progress.
 */

{cgidefs.i}

Define variable formCustNum     as character no-undo.
Define variable intCustNum      as integer no-undo.
Define variable ValidCustomer   as logical no-undo.


/*
 *  This function is similar to the url-encode WebSpeed function, but it
 *  specificly targets the unsafe characters for xml 1.0 as described in
 *  http://www.w3.org/TR/2004/REC-xml-20040204/#syntax
 */
FUNCTION xml-encode RETURNS CHARACTER
  (INPUT ipCharData AS CHARACTER):

  Define variable xml-unsafe  as character no-undo initial "<>&":U.
  Define variable Idx         as integer no-undo.
  Define variable AsciiValue  as integer no-undo.
  Define variable HexChars    as character no-undo initial "0123456789ABCDEF":U.

  /* Don't bother with blank or unknown  */
  if length( ipCharData ) = 0 or ipCharData = ? then
    return "":U.

  /* Loop through entire input string */
  assign Idx = 0.
  do while true:
    assign Idx = Idx + 1
           /* ASCII value of character using single byte codepage */
           AsciiValue = asc( substring( ipCharData, Idx, 1, "RAW":U ),
                             "1252", "1252" ).

    if AsciiValue le 31 or AsciiValue ge 127
       or index( xml-unsafe, chr( AsciiValue ) ) gt 0 then
    do:
      /* Replace character with %hh hexidecimal triplet */
      substring( ipCharData, Idx, 1, "RAW":U ) =
        "%":U +
        substring( HexChars, integer( truncate( AsciiValue / 16, 0 ) ) + 1,
                   1, "RAW":U) + /* high */
        substring( HexChars, AsciiValue modulo 16 + 1, 1, "RAW":U ). /* low */
      assign Idx = Idx + 2.   /* skip over hex triplet just inserted */
    end.
    if Idx = length(ipCharData,"RAW") then leave.
  end.

  return ipCharData.

END FUNCTION.  /**  xml-encode  **/


assign formCustNum = get-value("cnum")
       intCustNum  = ?.

/**  first, check to see if we even have an integer customer number  **/
assign intCustNum = integer( formCustNum ) NO-ERROR.

if intCustNum eq ? then
  ValidCustomer = no.
else
  ValidCustomer = can-find( Customer where Customer.CustNum eq intCustNum ).


{&OUT}
  '<?xml version="1.0" encoding="UTF-8" standalone="yes"?>' skip
  '<orders>' skip.

if ValidCustomer then
do:
  for each Order where Order.CustNum eq intCustNum no-lock:

    {&OUT}
      '  <order>' skip
      '    <ordernum>' Order.Ordernum '</ordernum>' skip
      '    <orderdate>' ( if Order.OrderDate eq ? then ''
                            else string( Order.OrderDate, '99/99/9999' ) )
          '</orderdate>' skip
      '    <shipdate>' ( if Order.ShipDate eq ? then ''
                           else string( Order.ShipDate, '99/99/9999' ) )
          '</shipdate>' skip
      '    <po>' xml-encode( Order.PO )  '</po>' skip
      '  </order>' skip.

  end.  /**  for each Order  **/
end.  /**  valid Customer  **/

{&OUT}
  '</orders>' skip.

